@@ -5,9 +5,10 @@ They may be useful in writing new implementations of these components, or for te
 """
 
 from abc import ABCMeta, abstractmethod, abstractproperty
+from .versioned_data_kind import VersionedDataKind
+from typing import Mapping, Callable, Any
 
-
-class FeatureStore(object):
+class FeatureStore:
     """
     Interface for a versioned store for feature flags and related objects received from LaunchDarkly.
     Implementations should permit concurrent access and updates.
@@ -15,93 +16,79 @@ class FeatureStore(object):
     An "object", for ``FeatureStore``, is simply a dict of arbitrary data which must have at least
     three properties: ``key`` (its unique key), ``version`` (the version number provided by
     LaunchDarkly), and ``deleted`` (True if this is a placeholder for a deleted object).
-    
+
     Delete and upsert requests are versioned: if the version number in the request is less than
     the currently stored version of the object, the request should be ignored.
-    
+
     These semantics support the primary use case for the store, which synchronizes a collection
     of objects based on update messages that may be received out-of-order.
     """
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def get(self, kind, key, callback=lambda x: x):
+    def get(self, kind: VersionedDataKind, key: str, callback: Callable[[Any], Any]=lambda x: x) -> Any:
         """
         Retrieves the object to which the specified key is mapped, or None if the key is not found
         or the associated object has a ``deleted`` property of True. The retrieved object, if any (a
         dict) can be transformed by the specified callback.
 
         :param kind: The kind of object to get
-        :type kind: VersionedDataKind
         :param key: The key whose associated object is to be returned
-        :type key: str
         :param callback: A function that accepts the retrieved data and returns a transformed value
-        :type callback: function
         :return: The result of executing callback
         """
 
     @abstractmethod
-    def all(self, kind, callback=lambda x: x):
+    def all(self, kind: VersionedDataKind, callback: Callable[[Any], Any]=lambda x: x) -> Any:
         """
         Retrieves a dictionary of all associated objects of a given kind. The retrieved dict of keys
         to objects can be transformed by the specified callback.
 
         :param kind: The kind of objects to get
-        :type kind: VersionedDataKind
         :param callback: A function that accepts the retrieved data and returns a transformed value
-        :type callback: function
-        :rtype: The result of executing callback
         """
 
     @abstractmethod
-    def init(self, all_data):
+    def init(self, all_data: Mapping[VersionedDataKind, Mapping[str, dict]]):
         """
         Initializes (or re-initializes) the store with the specified set of objects. Any existing entries
         will be removed. Implementations can assume that this set of objects is up to date-- there is no
         need to perform individual version comparisons between the existing objects and the supplied data.
 
         :param all_data: All objects to be stored
-        :type all_data: dict[VersionedDataKind, dict[str, dict]]
         """
 
     @abstractmethod
-    def delete(self, kind, key, version):
+    def delete(self, kind: VersionedDataKind, key: str, version: int):
         """
         Deletes the object associated with the specified key, if it exists and its version is less than
         the specified version. The object should be replaced in the data store by a
         placeholder with the specified version and a "deleted" property of TErue.
 
         :param kind: The kind of object to delete
-        :type kind: VersionedDataKind
         :param key: The key of the object to be deleted
-        :type key: str
         :param version: The version for the delete operation
-        :type version: int
         """
 
     @abstractmethod
-    def upsert(self, kind, item):
+    def upsert(self, kind: VersionedDataKind, item: dict):
         """
         Updates or inserts the object associated with the specified key. If an item with the same key
         already exists, it should update it only if the new item's version property is greater than
         the old one.
 
         :param kind: The kind of object to update
-        :type kind: VersionedDataKind
         :param item: The object to update or insert
-        :type feature: dict
         """
 
     @abstractproperty
-    def initialized(self):
+    def initialized(self) -> bool:
         """
         Returns whether the store has been initialized yet or not
-
-        :rtype: bool
         """
 
 
-class FeatureStoreCore(object):
+class FeatureStoreCore:
     """
     Interface for a simplified subset of the functionality of :class:`FeatureStore`, to be used
     in conjunction with :class:`ldclient.feature_store_helpers.CachingStoreWrapper`. This allows
@@ -112,34 +99,29 @@ class FeatureStoreCore(object):
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def get_internal(self, kind, key):
+    def get_internal(self, kind: VersionedDataKind, key: str) -> dict:
         """
         Returns the object to which the specified key is mapped, or None if no such item exists.
         The method should not attempt to filter out any items based on their deleted property,
         nor to cache any items.
 
         :param kind: The kind of object to get
-        :type kind: VersionedDataKind
         :param key: The key of the object
-        :type key: str
         :return: The object to which the specified key is mapped, or None
-        :rtype: dict
         """
 
     @abstractmethod
-    def get_all_internal(self, callback):
+    def get_all_internal(self, callback) -> Mapping[str, dict]:
         """
         Returns a dictionary of all associated objects of a given kind. The method should not attempt
         to filter out any items based on their deleted property, nor to cache any items.
 
         :param kind: The kind of objects to get
-        :type kind: VersionedDataKind
         :return: A dictionary of keys to items
-        :rtype: dict[str, dict]
         """
 
     @abstractmethod
-    def init_internal(self, all_data):
+    def init_internal(self, all_data: Mapping[VersionedDataKind, Mapping[str, dict]]):
         """
         Initializes (or re-initializes) the store with the specified set of objects. Any existing entries
         will be removed. Implementations can assume that this set of objects is up to date-- there is no
@@ -147,11 +129,10 @@ class FeatureStoreCore(object):
         data.
 
         :param all_data: A dictionary of data kinds to item collections
-        :type all_data: dict[VersionedDataKind, dict[str, dict]]
         """
 
     @abstractmethod
-    def upsert_internal(self, kind, item):
+    def upsert_internal(self, kind: VersionedDataKind, item: dict) -> dict:
         """
         Updates or inserts the object associated with the specified key. If an item with the same key
         already exists, it should update it only if the new item's version property is greater than
@@ -161,27 +142,22 @@ class FeatureStoreCore(object):
         `CachingStoreWrapper` will update the cache correctly).
 
         :param kind: The kind of object to update
-        :type kind: VersionedDataKind
         :param item: The object to update or insert
-        :type item: dict
         :return: The state of the object after the update
-        :rtype: dict
         """
 
     @abstractmethod
-    def initialized_internal(self):
+    def initialized_internal(self) -> bool:
         """
         Returns true if this store has been initialized. In a shared data store, it should be able to
         detect this even if initInternal was called in a different process, i.e. the test should be
         based on looking at what is in the data store. The method does not need to worry about caching
         this value; `CachingStoreWrapper` will only call it when necessary.
-
-        :rtype: bool
         """
 
 
 # Internal use only. Common methods for components that perform a task in the background.
-class BackgroundOperation(object):
+class BackgroundOperation:
 
     # noinspection PyMethodMayBeStatic
     def start(self):
@@ -197,10 +173,9 @@ class BackgroundOperation(object):
         pass
 
     # noinspection PyMethodMayBeStatic
-    def is_alive(self):
+    def is_alive(self) -> bool:
         """
         Returns whether the operation is alive or not
-        :rtype: bool
         """
         return True
 
@@ -213,15 +188,13 @@ class UpdateProcessor(BackgroundOperation):
     """
     __metaclass__ = ABCMeta
 
-    def initialized(self):
+    def initialized(self) -> bool:
         """
         Returns whether the update processor has received feature flags and has initialized its feature store.
-
-        :rtype: bool
         """
 
 
-class EventProcessor(object):
+class EventProcessor:
     """
     Interface for the component that buffers analytics events and sends them to LaunchDarkly.
     The default implementation can be replaced for testing purposes.
@@ -242,7 +215,7 @@ class EventProcessor(object):
         until a later time. However, calling ``stop()`` will synchronously deliver any events that were
         not yet delivered prior to shutting down.
         """
-    
+
     @abstractmethod
     def stop(self):
         """
@@ -250,7 +223,7 @@ class EventProcessor(object):
         """
 
 
-class FeatureRequester(object):
+class FeatureRequester:
     """
     Interface for the component that acquires feature flag data in polling mode. The default
     implementation can be replaced for testing purposes.
@@ -264,17 +237,16 @@ class FeatureRequester(object):
         pass
 
 
-class DiagnosticDescription(object):
+class DiagnosticDescription:
     """
     Optional interface for components to describe their own configuration.
     """
 
     @abstractmethod
-    def describe_configuration(self, config):
+    def describe_configuration(self, config) -> str:
         """
         Used internally by the SDK to inspect the configuration.
-        :param ldclient.config.Config config: the full configuration, in case this component depends on properties outside itself
+        :param config: the full configuration, in case this component depends on properties outside itself
         :return: a string describing the type of the component, or None
-        :rtype: string
         """
         pass
