@@ -4,7 +4,7 @@ from threading import Thread
 import time
 import uuid
 
-from ldclient.config import Config
+from ldclient.config import Config, HTTPConfig
 from ldclient.diagnostics import create_diagnostic_id, _DiagnosticAccumulator
 from ldclient.event_processor import DefaultEventProcessor
 from ldclient.util import log
@@ -13,7 +13,7 @@ from testing.proxy_test_util import do_proxy_tests
 from testing.stub_util import MockResponse, MockHttp
 
 
-default_config = Config()
+default_config = Config("fake_sdk_key")
 user = {
     'key': 'userkey',
     'name': 'Red'
@@ -69,6 +69,8 @@ class DefaultTestProcessor(DefaultEventProcessor):
     def __init__(self, **kwargs):
         if not 'diagnostic_opt_out' in kwargs:
             kwargs['diagnostic_opt_out'] = True
+        if not 'sdk_key' in kwargs:
+            kwargs['sdk_key'] = 'SDK_KEY'
         config = Config(**kwargs)
         diagnostic_accumulator = _DiagnosticAccumulator(create_diagnostic_id(config))
         DefaultEventProcessor.__init__(self, config, mock_http, diagnostic_accumulator = diagnostic_accumulator)
@@ -215,7 +217,7 @@ def test_two_events_for_same_user_only_produce_one_index_event():
             'kind': 'feature', 'key': 'flagkey', 'version': 11, 'user': user,
             'variation': 1, 'value': 'value', 'default': 'default', 'trackEvents': True
         }
-        e1 = e0.copy();
+        e1 = e0.copy()
         ep.send_event(e0)
         ep.send_event(e1)
 
@@ -232,8 +234,8 @@ def test_new_index_event_is_added_if_user_cache_has_been_cleared():
             'kind': 'feature', 'key': 'flagkey', 'version': 11, 'user': user,
             'variation': 1, 'value': 'value', 'default': 'default', 'trackEvents': True
         }
-        e1 = e0.copy();
-        ep.send_event(e0);
+        e1 = e0.copy()
+        ep.send_event(e0)
         time.sleep(0.2)
         ep.send_event(e1)
 
@@ -278,33 +280,6 @@ def test_event_can_be_both_tracked_and_debugged():
         check_feature_event(output[2], e, True, user)
         check_summary_event(output[3])
 
-def test_debug_mode_does_not_expire_if_both_client_time_and_server_time_are_before_expiration_time():
-    with DefaultTestProcessor() as ep:
-        # Pick a server time that slightly different from client time
-        server_time = now() + 1000
-
-        # Send and flush an event we don't care about, just to set the last server time
-        mock_http.set_server_time(server_time)
-        ep.send_event({ 'kind': 'identify', 'user': { 'key': 'otherUser' }})
-        flush_and_get_events(ep)
-
-        # Now send an event with debug mode on, with a "debug until" time that is further in
-        # the future than both the client time and the server time
-        debug_until = server_time + 10000
-        e = {
-            'kind': 'feature', 'key': 'flagkey', 'version': 11, 'user': user,
-            'variation': 1, 'value': 'value', 'default': 'default',
-            'trackEvents': False, 'debugEventsUntilDate': debug_until
-        }
-        ep.send_event(e)
-
-        # Should get a summary event only, not a full feature event
-        output = flush_and_get_events(ep)
-        assert len(output) == 3
-        check_index_event(output[0], e, user)
-        check_feature_event(output[1], e, True, user)  # debug event
-        check_summary_event(output[2])
-
 def test_debug_mode_expires_based_on_client_time_if_client_time_is_later_than_server_time():
     with DefaultTestProcessor() as ep:
         # Pick a server time that is somewhat behind the client time
@@ -558,7 +533,7 @@ def test_will_still_send_after_500_error():
     verify_recoverable_http_error(500)
 
 def test_does_not_block_on_full_inbox():
-    config = Config(events_max_pending=1)  # this sets the size of both the inbox and the outbox to 1
+    config = Config("fake_sdk_key", events_max_pending=1)  # this sets the size of both the inbox and the outbox to 1
     ep_inbox_holder = [ None ]
     ep_inbox = None
 
